/*
 * The contents of this file are subject to the terms
 * of the Common Development and Distribution License
 * (the License).  You may not use this file except in
 * compliance with the License.
 *
 * You can obtain a copy of the license at
 * https://glassfish.dev.java.net/public/CDDLv1.0.html or
 * glassfish/bootstrap/legal/CDDLv1.0.txt.
 * See the License for the specific language governing
 * permissions and limitations under the License.
 *
 * When distributing Covered Code, include this CDDL
 * Header Notice in each file and include the License file
 * at glassfish/bootstrap/legal/CDDLv1.0.txt.
 * If applicable, add the following below the CDDL Header,
 * with the fields enclosed by brackets [] replaced by
 * you own identifying information:
 * "Portions Copyrighted [year] [name of copyright owner]"
 *
 * Copyright 2006 Sun Microsystems, Inc. All rights reserved.
 */
package com.sun.grizzly.http;

import java.nio.channels.SelectionKey;
import java.io.IOException;
import com.sun.grizzly.Context;
import com.sun.grizzly.TCPSelectorHandler;
import com.sun.grizzly.util.Copyable;
import java.util.concurrent.ConcurrentLinkedQueue;

/**
 * <code>SelectorHandler</code> implementation <code>SelectorThread</code> 
 * passes to <code>Controller</code>. It is very similar to 
 * <code>TCPSelectorHandler</code>, however has some difference in preSelect()
 * processing
 * 
 * @author Jeanfrancois Arcand
 * @author Alexey Stashok
 */
public class SelectorThreadHandler extends TCPSelectorHandler {
    private SelectorThread selectorThread;
    
    @Override
    public void copyTo(Copyable copy) {
        super.copyTo(copy);
        SelectorThreadHandler copyHandler = (SelectorThreadHandler) copy;
        copyHandler.selectorThread = selectorThread;
    }

    public void setSelectorThread(SelectorThread selectorThread) {
        this.selectorThread = selectorThread;
    }
    
    @Override
    public void preSelect(Context controllerCtx) throws IOException {
        if (selector != null) {
            if (opReadToRegister == null) {
                opReadToRegister = new ConcurrentLinkedQueue<SelectionKey>();
            }

            SelectionKey key;
            int size = opReadToRegister.size();
            long currentTime = 0L;
            if (size > 0) {
                currentTime = (Long) System.currentTimeMillis();
            }

            for (int i = 0; i < size; i++) {
                key = opReadToRegister.poll();
                key.interestOps(key.interestOps() | SelectionKey.OP_READ);

                if (key.attachment() == null) {
                    key.attach(currentTime);
                }
                
                selectorThread.keepAliveCounter.trap(key);
            }
        } else {
            super.preSelect(controllerCtx);
        }
    }
}
